import datetime

def sync_facebook_test_user(sender, **kwargs):
    from django.utils import importlib
    models = importlib.import_module('fandjango.models')

    # Split out the test user's name
    name_parts = kwargs['test_user'].name.split(" ")
    first_name = name_parts[0]
    last_name = name_parts[-1]
    middle_name = None
    if len(name_parts) < 2:
        middle_name = " ".join(name_parts[1:-1])

    # Update the Fandjango user with the same name, creating a new one if need be
    # Either don't create a new one or delete the user if the app isn't installed for the test user.
    user_installed_app = kwargs['test_user'].access_token is not None and len(kwargs['test_user'].access_token) > 0
    try:
        db_user = models.User.objects.get(
            first_name=first_name,
            middle_name=middle_name,
            last_name=last_name)
    except models.User.DoesNotExist:
        if user_installed_app:
            db_user = models.User()
            db_user.first_name = first_name
            db_user.middle_name = middle_name
            db_user.last_name = last_name
            db_user.oauth_token = models.OAuthToken.objects.create(token="", issued_at=datetime.datetime.now())
        else:
            db_user = None
    if user_installed_app and db_user is not None:
        db_user.facebook_id = int(kwargs['test_user'].facebook_id)
        db_user.oauth_token.token = kwargs['test_user'].access_token
        db_user.save()
        db_user.oauth_token.save()
    elif db_user is not None:
        db_user.delete()

def setup_facebook_test_client(sender, **kwargs):
    kwargs['client'].cookies['signed_request'] = kwargs['signed_request']



import os
import re
import urllib
import urlparse

from django.http import HttpResponseRedirect
from facetools.url import translate_url_to_facebook_url, facebook_redirect

GET_REDIRECT_PARAM = 'facebook_redirect'

class FacebookRedirectMiddleware(object):
    def process_request(self, request):
        # Look for GET parameter, and redirect if you find it
        redirect_url = request.GET.get(GET_REDIRECT_PARAM)
        if redirect_url:
            return facebook_redirect(redirect_url)
        return None

    def process_response(self, request, response):
        if type(response) == HttpResponseRedirect:
            facebook_url = translate_url_to_facebook_url(response['Location'])
            if facebook_url != response['Location']:
                get_params = urllib.urlencode({GET_REDIRECT_PARAM: facebook_url})
                prefix = "?"
                insert_position = len(response['Location'])
                if '?' in response['Location']:
                    prefix = "&"
                if "#" in response['Location']:
                    insert_position = response['Location'].index("#")
                response['Location'] = response['Location'][:insert_position] + \
                                       prefix + get_params + \
                                       response['Location'][insert_position:]
        return response

class FandjangoIntegrationMiddleware(object):

    def process_response(self, request, response):
        if self.should_process_response(response):
            # Get the oauth url fandjango is redirecting the user to
            oauth_url = self.get_oauth_url(response.content, "window.parent.location =", ';')

            # Update the oauth url so that it's url it goes to after
            # the user authorizes the app is a translated facebook url
            redirect_uri = urlparse.parse_qs(urlparse.urlparse(oauth_url).query)['redirect_uri'][0]
            path = urlparse.urlparse(redirect_uri).path
            path = '/' + "/".join(path.split("/")[2:])
            if not path.endswith("/"): path = path + "/"
            new_url = translate_url_to_facebook_url(path)

            # Replace the old url with the new one
            start_token = "redirect_uri="
            start = oauth_url.index(start_token) + len(start_token)
            end = oauth_url.index("&", start)
            new_oauth_url = oauth_url.replace(oauth_url[start:end], urllib.quote_plus(new_url))
            response.content = response.content.replace(oauth_url, new_oauth_url)

        return response

    def should_process_response(self, response):
        # We're going to compare this response's content to Fandjango's template
        # for authorization and see if they are the same.  First we need to find
        # the dynamic content in the template and in this response's html
        try:
            oauth_url_to_remove = self.get_oauth_url(response.content, 'window.parent.location = ', ';')
        except:
            return False
        if response.content.count(oauth_url_to_remove) != 2:
            return False
        import fandjango
        template_path = os.path.join(os.path.dirname(fandjango.__file__), "templates/fandjango/authorize_application.html")
        with open(template_path) as fandjango_temlate_file:
            fandjango_template_content = fandjango_temlate_file.read()
        template_tags_to_remove = re.findall("\{\{.*url\|safe \}\}", fandjango_template_content)
        if len(template_tags_to_remove) != 2:
            return False

        # Strip out the dynamic content
        response_template_content = response.content.replace(oauth_url_to_remove, "")
        for template_tag_to_remove in template_tags_to_remove:
            fandjango_template_content = fandjango_template_content.replace(template_tag_to_remove, "")

        # If the response minus its dynamic content is identical to Fandjango's
        # template minus its dynamic content then we should process it
        return response_template_content == fandjango_template_content

    def get_oauth_url(self, content, start_token, end_token):
        start = content.index(start_token) + len(start_token)
        end = content.index(end_token, start)
        return content[start:end].strip()[1:-1] # remove any whitespace and quotes around the url


import os
import sys
from facetools import json

from django.core import management
from django.core.management.base import AppCommand, BaseCommand
from django.conf import settings
import requests

from facetools.test.testusers import _create_test_user, _friend_test_users, _create_test_user_on_facebook
from facetools.signals import sync_facebook_test_user
from facetools.models import TestUser
from facetools.common import _get_facetools_test_fixture_name


class Command(AppCommand):
    help = 'Creates the facebook test users defined in each app in the project.'

    def handle_app(self, app, **options):
        app_name = '.'.join(app.__name__.split('.')[0:-1])

        test_users = _get_facetools_test_users(app_name)
        existing_facebook_test_users = _get_existing_facebook_test_users()
        existing_facetool_test_users = [u.name for u in TestUser.objects.all()]
        existing_test_users = set(existing_facebook_test_users.keys() + existing_facetool_test_users)

        # Create any test users on facebook their corresponding User models in facetools
        # that don't exist on facebook yet.
        for test_user in test_users:
            _print('Syncing %s' % test_user['name'])

            # Add user to facebook and local database
            if test_user['name'] not in existing_test_users:
                _create_test_user(
                    app_installed = test_user.get('installed', True),
                    name          = test_user['name'],
                    permissions   = test_user.get('permissions')
                )

            # or add test user to facebook and sync with existing test user in the local database
            elif test_user['name'] not in existing_facebook_test_users:
                facebook_response_data = _create_test_user_on_facebook(
                    app_installed = test_user.get('installed', True),
                    name          = test_user['name'],
                    permissions   = test_user.get('permissions')
                )
                facetools_user = TestUser.objects.get(name=test_user['name'])
                facetools_user.facebook_id = facebook_response_data['id']
                facetools_user.access_token = facebook_response_data.get('access_token')
                facetools_user.login_url = facebook_response_data.get('login_url')
                facetools_user.save()

            # or add test user to the local database using the test user's data on facebook
            elif test_user['name'] not in existing_facetool_test_users:
                facebook_data = existing_facebook_test_users[test_user['name']]
                TestUser.objects.create(
                    name         = test_user['name'],
                    facebook_id  = facebook_data['id'],
                    access_token = facebook_data.get('access_token'),
                    login_url    = facebook_data.get('login_url')
                )

            # or sync the existing user with the latest facebook information
            else:
                facebook_data = existing_facebook_test_users[test_user['name']]
                facetools_user = TestUser.objects.get(name=test_user['name'])
                facetools_user.facebook_id = facebook_data['id']
                facetools_user.access_token = facebook_data.get('access_token')
                facetools_user.login_url = facebook_data.get('login_url')
                facetools_user.save()

            sync_facebook_test_user.send(sender=None, test_user=TestUser.objects.get(name=test_user['name']))

        _create_test_user_friendships(test_users)
        _create_test_user_fixture(app, app_name, test_users)

def _get_facetools_test_users(app_name, test_user_module_name='facebook_test_users'):
    """Get the dictionary of facebook test users for the app, throwing an error if the app
    doesn't have any defined."""
    try:
        _temp = __import__(app_name, globals(), locals(), [test_user_module_name])
        facetools_test_users = _temp.facebook_test_users.facebook_test_users
        if callable(facetools_test_users):
            facetools_test_users = facetools_test_users()
    except ImportError:
        raise Exception("Error: %s doesn't have a module called %s" % (app_name, test_user_module_name))

    # Ensure no test users share the same name
    facetools_test_names = set([u['name'] for u in facetools_test_users])
    if len(facetools_test_names) != len(facetools_test_users):
        raise Exception("Error: Duplicate names found in %s for %s" % (test_user_module_name, app_name))

    return facetools_test_users

def _get_existing_facebook_test_users(app_id=settings.FACEBOOK_APPLICATION_ID, app_secret=settings.FACEBOOK_APPLICATION_SECRET_KEY):
    """
    Get the facebook data for each test user defined for the app.
    """
    existing_facebook_test_users = {}
    app_access_token = '%s|%s' % (app_id, app_secret)
    test_users_url = "https://graph.facebook.com/%s/accounts/test-users?access_token=%s" % (app_id, app_access_token)
    for attempts in range(3, 0, -1):
        response = requests.get(test_users_url)
        try: users_response_data = json.loads(response.content)
        except: users_response_data = None
        if response.status_code != 200 or users_response_data is None or users_response_data == False or 'error' in users_response_data:
            if attempts > 0:
                continue
            try:
                raise Exception("Error retrieving facebook app's test users: %s" % users_response_data['error']['message'])
            except:
                try:
                    raise Exception("Error retrieving facebook app's test users: status_code=%s, content=\"%s\"" % (response.status_code, response.content))
                except:
                    raise Exception("Error retrieving facebook app's test users: status_code=%s" % response.status_code)
        else:
            break

    test_user_url = "https://graph.facebook.com/%s?access_token=%s"
    for test_user in users_response_data['data']:
        user_response_data = json.loads(requests.get(test_user_url % (test_user['id'], app_access_token)).content)
        if user_response_data == False:
            # skip invalid users defined on facebook
            continue
        if 'error' in user_response_data:
            raise Exception("Error retrieving data for %s: %s" % (test_user['id'], user_response_data['error']['message']))
        elif 'name' in user_response_data:
            user_response_data['access_token'] = test_user.get('access_token')
            user_response_data['login_url'] = test_user.get('login_url')
            existing_facebook_test_users[user_response_data['name']] = user_response_data

    return existing_facebook_test_users

def _get_test_user_relationships(test_users):
    """
    Takes a facebook_test_users dict and returns a list of relationships, with
    each item being a set of 2 names that should be friends.
    """
    relationships = []
    for test_user in test_users:
        if 'friends' in test_user:
            for friend in test_user['friends']:
                relationships.append(set([test_user['name'], friend]))
    no_dupes = []
    for relationship in relationships:
        if relationship not in no_dupes:
            no_dupes.append(relationship)
    return no_dupes

def _get_app_fixture_directory(app):
    """
    Gets the fixture directory for the app, creating it if it doesn't exist.
    """
    app_dir = os.path.dirname(app.__file__)
    fixture_dir = os.path.join(app_dir, "fixtures")
    if os.path.exists(fixture_dir):
        if not os.path.isdir(fixture_dir):
            raise IOError("Can't create a fixture directory for %s.  File already exists." % app.__package__)
    else:
        os.mkdir(fixture_dir)
    return fixture_dir

def _print(s):
    if sys.argv[1] != 'test':
        print s

def _create_test_user_friendships(test_users):
    """
    Create the friendships defined between all the test users
    """
    friendships = [list(r) for r in _get_test_user_relationships(test_users)]
    for friendship in friendships:
        friendship = list(friendship)
        _print('Friending %s and %s' % (friendship[0], friendship[1]))
        _friend_test_users(friendship[0], friendship[1])

def _create_test_user_fixture(app, app_name, test_users):
    """
    Create the fixture for the test users
    """
    fixture_file_path = os.path.join(_get_app_fixture_directory(app),
                                     _get_facetools_test_fixture_name(app_name))
    _print('Creating fixture for test users at %s' % fixture_file_path)

    # Create the initial fixture
    old_stdout = sys.stdout
    try:
        sys.stdout = open(fixture_file_path,'w')
        management.call_command('dumpdata', 'facetools', indent=4)
    finally:
        if sys.stdout != old_stdout:
            f=sys.stdout
            sys.stdout=old_stdout
            f.close()

    # Clean up the fixture so it only has this app's test users
    fixture_content = open(fixture_file_path).read()
    with open(fixture_file_path, 'w') as fixture_file:
        fixture_file.write(_clean_test_user_fixture(fixture_content, test_users))

def _clean_test_user_fixture(fixture_content, test_users):
    """
    Removes any test users in a fixture that aren't defined in the app's test users
    """
    fixture_names = set([u['pk'] for u in json.loads(fixture_content)])
    actual_names = set([u['name'] for u in test_users])
    names_to_remove = fixture_names - actual_names
    for name_to_remove in names_to_remove:
        name_index = fixture_content.index(name_to_remove)
        start = fixture_content.rindex('{', 0, name_index)      # starting bracket for json entry
        end_fields = fixture_content.index('}', name_index) + 1 # ending bracket for 'fields' dict in json entry
        end = fixture_content.find('{', end_fields)             # starting bracket for next json entry
        if end == -1:
            end = fixture_content.find('}', end_fields) + 1     # ending bracket for json entry
            start = fixture_content.rindex(',', 0, start)       # comma of previous json entry (i.e. { ... }, )
        fixture_content = fixture_content[:start] + fixture_content[end:]
    return fixture_content


import types

from django.template.defaulttags import URLNode, url
from django import template

from facetools.url import translate_url_to_facebook_url

register = template.Library()

def facebook_url(parser, token):
    """
    Works exactly like the default url tag, except the final url is replaced
    with its facebook canvas equivalent if it's in the canvas app.

    Refer to https://docs.djangoproject.com/en/1.3/ref/templates/builtins/#url
    for more information on how to use this tag.
    """

    # The {% url %} tag's function generates a URLNode.  The URLNode's render
    # function handles generating the proper url.  We're going to move the
    # the render method and replace it with a new render method.  The new
    # method will call the old one and then adjust the url before returning.
    URLNode_obj = url(parser, token)
    URLNode_obj.old_render = URLNode_obj.render
    URLNode_obj.render = types.MethodType(
        replacement_URLNode_render_method,
        URLNode_obj,
        URLNode)
    return URLNode_obj
facebook_url = register.tag(facebook_url)

def replacement_URLNode_render_method(self, context):
    """
    Replacement for the URLNode's render method.  It's monkey patched into
    a URLNode instance in the facebook_url tag.
    """
    # NOTE: Pretend this method exists inside the URLNode class (because when it's called it will be)
    url_str = self.old_render(context)
    if url_str:
        url_str = translate_url_to_facebook_url(url_str)
    return url_str



from urlparse import urlparse

from django.http import HttpResponse
from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.conf import settings

def translate_url_to_facebook_url(url):
    """
    Converts a url on the canvas path to it's facebook equivalent.  Requires
    FACEBOOK_CANVAS_URL and FACEBOOK_CANVAS_PAGE to be set in your Django
    settings.

    ex: http://localhost:8000/canvas/page/ becomes https://apps.facebook.com/my_facebook_app/page/
    """
    url_parts = urlparse(url)
    canvas_url = settings.FACEBOOK_CANVAS_URL
    if canvas_url.endswith('/'): canvas_url = canvas_url[:-1]
    canvas_parts = urlparse(canvas_url)

    # Dont convert the url if its not a canvas url
    if url_parts.scheme and not canvas_url.startswith('%s://%s' % (url_parts.scheme, url_parts.netloc)):
        return url
    if url_parts.netloc and not (canvas_url.startswith('http://%s' % url_parts.netloc) or canvas_url.startswith('https://%s' % url_parts.netloc)):
        return url
    if url_parts.path and not url_parts.path.startswith('/'):
        net_loc = url_parts.path
        if '/' in net_loc: net_loc = net_loc[:net_loc.index('/')]
        if not (canvas_url.startswith('http://%s' % net_loc) or canvas_url.startswith('https://%s' % net_loc)):
            return url
    if not url_parts.path.startswith(canvas_parts.path):
        return url

    facebook_url = settings.FACEBOOK_CANVAS_PAGE
    if facebook_url.endswith('/'): facebook_url = facebook_url[:-1]
    start = len(url_parts.scheme) + len(url_parts.netloc)
    if url_parts.scheme.strip():
        start += len("://")
    start = url.index(canvas_parts.path, start) + len(canvas_parts.path)
    full_new_path = url[start:]
    return '%s%s' % (facebook_url, full_new_path)

def facebook_reverse(*args, **kwargs):
    """
    Drop in replacement for Django's reverse function.  Modifies
    the url so it accounts for the facebook canvas url::

        >>> reverse('myapp:page')
        /canvas/page/
        >>> facebook_reverse('myapp:page')
        https://apps.facebook.com/myapp/page/

    Refer to https://docs.djangoproject.com/en/1.3/topics/http/urls/#reverse
    for more information on the arguments ``facebook_reverse`` can take.
    """
    url = reverse(*args, **kwargs)
    return translate_url_to_facebook_url(url)

def facebook_redirect(to, skip_replace=False, *args, **kwargs):
    """
    Drop in replacement for Django's redirect function.  Instead of returning a
    redirect using HTTP status codes and header values, it returns a regular HTML
    response with an empty body and a javascript fragment that sets the
    location variable in the DOM::

        <script type="text/javascript">
            top.location.href="https://apps.facebook.com/myapp/page/";
        </script>

    This allows us to redirect inside a Facebook
    IFrame to outside facebook pages (like authentication) and causes the
    users url in the address bar to update when redirecting to another internal page.

    Refer to https://docs.djangoproject.com/en/1.3/topics/http/shortcuts/#redirect
    for more information on the agruments the ``facebook_redirect`` can take.
    """
    html_template = """
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
            "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
        <script type="text/javascript">
            top.location.href="%(url)s";
        </script>
    </head>
    <body>
        <noscript>
            Redirecting to <a href="%(url)s" target="_top">%(url)s</a>.  Please <a href="%(url)s" target="_top">click here</a> if you aren't redirected.
        </noscript>
    </body>
    </html>
    """

    redirect_response = redirect(to, *args, **kwargs)
    url = redirect_response['Location']
    if not skip_replace:
        url = translate_url_to_facebook_url(url)

    return HttpResponse(html_template % {'url':url})



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'TestUser'
        db.create_table('facetools_testuser', (
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255, primary_key=True)),
            ('facebook_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('access_token', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
        ))
        db.send_create_signal('facetools', ['TestUser'])


    def backwards(self, orm):
        
        # Deleting model 'TestUser'
        db.delete_table('facetools_testuser')


    models = {
        'facetools.testuser': {
            'Meta': {'object_name': 'TestUser'},
            'access_token': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'facebook_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'primary_key': 'True'})
        }
    }

    complete_apps = ['facetools']



import urllib
import time

from facetools import json
from django.conf import settings
from facetools.common import _get_app_access_token, _create_permissions_string
from facetools.models import TestUser
import requests

class CreateTestUserError(Exception): pass
class DeleteTestUserError(Exception): pass
class NotATestUser(Exception): pass

# -------------------------------------------------------------------------------------
# Functions for creating test users on facebook and in facetools
# -------------------------------------------------------------------------------------

def _create_test_user(app_installed=True, name=None, permissions=None, access_token=None):
    """
    Creates a test user on facebook and a corresponding User and OAuthToken object in the database.
    Friends can be set with set_test_user_friends after all test users are created.  Assumes
    test user with name doesn't already exist.
    """
    facebook_response_data = _create_test_user_on_facebook(
        app_installed=app_installed,
        name=name,
        permissions=permissions,
        access_token=access_token
    )
    _create_test_user_in_facetools(name, facebook_response_data)

def _create_test_user_on_facebook(app_installed=True, name=None, permissions=None, access_token=None):
    """
    Creates a test user on facebook.  Returns a dict of the json response from facebook.
    """
    test_user_template = "https://graph.facebook.com/%s/accounts/test-users?installed=%s&permissions=%s&method=post&access_token=%s"

    # Generate the request URL
    if app_installed == True:
        app_installed = "true"
    else:
        app_installed = "false"
    if permissions is None:
        permissions = settings.FACEBOOK_APPLICATION_INITIAL_PERMISSIONS
    permissions = _create_permissions_string(permissions)
    if access_token is None:
        access_token = _get_app_access_token()
    test_user_url = test_user_template % (settings.FACEBOOK_APPLICATION_ID, app_installed, permissions, access_token)
    if name:
        test_user_url = '%s&name=%s' % (test_user_url,urllib.quote(name))

    # Request a new test user from facebook
    for attempts in range(3, 0, -1):
        r = requests.get(test_user_url)
        try: data = json.loads(r.content)
        except: data = None
        if r.status_code != 200 or data is None or data == False or 'error' in data:
            if attempts > 0:
                continue
            try:
                raise CreateTestUserError(data['error']['message'])
            except:
                try:
                    raise CreateTestUserError("Request to create test user failed (status_code=%s and content=\"%s\")" % (r.status_code, r.content))
                except:
                    raise CreateTestUserError("Request to create test user failed (status_code=%s)" % r.status_code)
        return data
    raise CreateTestUserError("Request to create test user failed")

def _create_test_user_in_facetools(name, facebook_data):
    # Add the user to the test user table
    if 'id' in facebook_data:
        try:
            test_user = TestUser.objects.get(facebook_id=int(facebook_data['id']))
        except TestUser.DoesNotExist:
            test_user = None
        if test_user is None:
            assert facebook_data['id'] is not None # This should never happen, so please freak out when it does
            TestUser.objects.create(
                name=name,
                facebook_id=str(facebook_data['id']),
                access_token=facebook_data.get('access_token'),
                login_url=facebook_data.get('login_url')
            )
        else:
            test_user.access_token = facebook_data.get('access_token')
            test_user.login_url = facebook_data.get('login_url')
            test_user.save()
    else:
        raise Exception("Invalid facebook user data")

def _delete_test_user_on_facebook(test_user):
    delete_url_template = "https://graph.facebook.com/%s?method=delete&access_token=%s"
    delete_user_url = delete_url_template % (test_user.facebook_id, _get_app_access_token())
    r = requests.delete(delete_user_url)
    if not isinstance(r.content, basestring):
        raise DeleteTestUserError("Error deleting user %s (%s) from facebook: Facebook returned invalid response" % (test_user.name, test_user.facebook_id, r.content))
    if r.content.strip().lower() != "true":
        if r.content.strip().lower() == "false":
            raise DeleteTestUserError("Error deleting user %s (%s) from facebook: Facebook returned false" % (test_user.name, test_user.facebook_id, r.content))
        else:
            try:
                raise DeleteTestUserError("Error deleting user %s (%s) from facebook: %s" % (test_user.name, test_user.facebook_id, json.loads(r.content)['error']['message']))
            except:
                raise DeleteTestUserError("Error deleting user %s (%s) from facebook: %s" % (test_user.name, test_user.facebook_id, r.content))

# -------------------------------------------------------------------------------------
# Functions for creating friends between test users
# -------------------------------------------------------------------------------------

def _friend_test_users(user, friend):
    """
    Makes two users friends. user friend a list of other test users.  The list
    of friends can be a list containing either a string of the friends
    full name, or the User object to friend.
    """
    user = TestUser.objects.get(name=user)
    friend = TestUser.objects.get(name=friend)
    response = requests.get("https://graph.facebook.com/%s/friends/%s?method=post&access_token=%s" %
            (user.facebook_id, friend.facebook_id, user.access_token))
    __handle_friend_test_users_error(user, friend, response)
    response = requests.get("https://graph.facebook.com/%s/friends/%s?method=post&access_token=%s" %
            (friend.facebook_id, user.facebook_id, friend.access_token))
    __handle_friend_test_users_error(user, friend, response)

def __handle_friend_test_users_error(user, friend, response):
    error_message = "Failed to friend %s with %s" % (user.name, friend.name)
    try: rdata = json.loads(response.content)
    except: rdata = False

    if rdata == False:
        raise CreateTestUserError(error_message)
    elif type(rdata) is not bool and 'error' in rdata:
        if not __already_friends(rdata):
            try:
                error_message += " (%s)" % rdata['error']['message']
            finally:
                raise CreateTestUserError(error_message)
    elif response.status_code != 200:
        raise CreateTestUserError(error_message)

def __already_friends(response_data):
    are_already_friends = False
    if 'error' in response_data and 'message' in response_data['error']:
        if 'You are already friends with this user' in response_data['error']['message']:
            are_already_friends = True
    return are_already_friends



from django.db import models

class TestUser(models.Model):
    """
    A test user on facebook.
    """
    name = models.CharField(max_length=255, primary_key=True)
    facebook_id = models.CharField(max_length=255, null=True, blank=True)
    access_token = models.CharField(max_length=255, null=True, blank=True)
    login_url = models.CharField(max_length=255, null=True, blank=True)

    def delete(self, *args, **kwargs):
        from facetools.test.testusers import _delete_test_user_on_facebook # avoiding circular dependency
        if self.facebook_id:
            for attempts in range(3, 0, -1):
                try:
                    _delete_test_user_on_facebook(self)
                    break
                except:
                    if attempts <= 0:
                        raise
        super(TestUser, self).delete(*args, **kwargs)

    def __unicode__(self):
        return self.name




from datetime import datetime
import base64
import hmac
import hashlib
import time

from django.conf import settings
from facetools import json

def _get_app_access_token(app_id=None, app_secret=None):
    """
    Creates the app access token using either specified Facebook app settings
    or the FACEBOOK_XXX variables defined in settings.py

    Arguments:
    app_id -- The application ID.  Defaults to FACBEOOK_APPLICATION_ID from settings
    app_secret -- The app's secret key.  Defaults to FACEBOOK_APPLICATION_SECRET_KEY from settings.
    """
    if app_id is None:
        app_id = settings.FACEBOOK_APPLICATION_ID
    if app_secret is None:
        app_secret = settings.FACEBOOK_APPLICATION_SECRET_KEY
    return '%s|%s' % (app_id, app_secret)

def _create_permissions_string(permission_list):
    """
    Creates an list of individual facebook permissions
    and makes a comma seperated string of permissions.
    """
    return ','.join(permission_list).replace(" ", "")

def _get_facetools_test_fixture_name(app_name):
    return "facetools_test_users_%s.json" % app_name

# ---------------------------------------------------------------------
# Following code originally by Reik Schatz. Taken from Fandjango. Thanks Reik!
# http://javasplitter.blogspot.com/
# https://github.com/reikje
# ---------------------------------------------------------------------

def _parse_signed_request(signed_request, app_secret):
    """Return dictionary with signed request data."""
    try:
        l = signed_request.split('.', 2)
        encoded_sig = str(l[0])
        payload = str(l[1])
    except IndexError:
        raise ValueError("Signed request malformed")

    sig = base64.urlsafe_b64decode(encoded_sig + "=" * ((4 - len(encoded_sig) % 4) % 4))
    data = base64.urlsafe_b64decode(payload + "=" * ((4 - len(payload) % 4) % 4))

    data = json.loads(data)

    if data.get('algorithm').upper() != 'HMAC-SHA256':
        raise ValueError("Signed request is using an unknown algorithm")
    else:
        expected_sig = hmac.new(app_secret, msg=payload, digestmod=hashlib.sha256).digest()

    if sig != expected_sig:
        raise ValueError("Signed request signature mismatch")
    else:
        return data

def _create_signed_request(app_secret, user_id=1, issued_at=None, oauth_token=None, expires=None, app_data=None, page=None, add_default_data=True):
    """
    Returns a string that is a valid signed_request parameter specified by Facebook
    see: http://developers.facebook.com/docs/authentication/signed_request/

    Arguments:
    app_secret -- the secret key that Facebook assigns to each Facebook application
    user_id -- optional a long representing the Facebook user identifier (UID) of a user
    issued_at -- optional an int or a datetime representing a timestamp when the request was signed
    oauth_token -- optional a String token to pass in to the Facebook graph api
    expires -- optional an int or a datetime representing a timestamp at which the oauth token expires
    app_data -- optional a dict containing additional application data
    page -- optional a dict having the keys id (string), liked (boolean) if the user has liked the page and optionally admin (boolean) if the user is an admin of that page.

    Regardless of which arguments are given, the encoded JSON object will always contain the following properties:
        -- user_id
        -- algorithm
        -- issued_at

    Examples:
        _create_signed_request(FACEBOOK_APPLICATION_SECRET_KEY)
        _create_signed_request(FACEBOOK_APPLICATION_SECRET_KEY, user_id=199)
        _create_signed_request(FACEBOOK_APPLICATION_SECRET_KEY, user_id=199, issued_at=1254459600)

    """
    payload = {
        'user_id': user_id,
        'algorithm': 'HMAC-SHA256',
    }
    if add_default_data:
        payload['expires'] = 0
        payload['user'] = {
            'country': 'us',
            'locale': 'en_US',
            'age': {
                'min': 0,
                'max': 150
            }
        }

    value = int(time.time())
    if issued_at is not None and (isinstance(issued_at, datetime) or isinstance(issued_at, int)):
        value = issued_at
        if isinstance(issued_at, datetime):
            value = int(time.mktime(issued_at.timetuple()))
    payload['issued_at'] = value

    if oauth_token is not None:
        payload['oauth_token'] = oauth_token
    if expires is not None and (isinstance(expires, datetime) or isinstance(expires, int)):
        value = expires
        if isinstance(expires, datetime):
            value = int(time.mktime(expires.timetuple()))
        payload['expires'] = value
    if app_data is not None and isinstance(app_data, dict):
        payload['app_data'] = app_data
    if page is not None and isinstance(page, dict):
        payload['page'] = page

    return _create_signed_request_parameter(app_secret, json.dumps(payload))

def _prepend_signature(app_secret, payload):
        """
            Returns a SHA256 signed and base64 encoded signature based on the given payload

            Arguments:
            app_secret -- the secret key that Facebook assigns to each Facebook application
            payload -- a base64url encoded String
        """
        dig = hmac.new(app_secret, msg=payload, digestmod=hashlib.sha256).digest()
        dig = base64.urlsafe_b64encode(dig)
        return dig

def _create_signed_request_parameter(app_secret, payload):
        """
            Returns a String value usable as the Facebook signed_request parameter. The String will be based on the given payload.
            The signed_request parameter is the concatenation of a HMAC SHA-256 signature string, a period (.), and a
            the base64url encoded payload.

            Arguments:
            app_secret -- the secret key that Facebook assigns to each Facebook application
            payload -- a JSON formatted String
        """
        base64_encoded_payload = base64.urlsafe_b64encode(payload)
        return _prepend_signature(app_secret, base64_encoded_payload) + "." + base64_encoded_payload

# ---------------------------------------------------------------------
# End snatched code by Reik Schatz.
# ---------------------------------------------------------------------


#!/usr/bin/env python

from distutils.core import setup

import facetools

def get_long_description():
    """
    Return the contents of the README file.
    """
    try:
        return open('README.rst').read()
    except:
        pass  # Required to install using pip (won't have README then)

setup(
    name = 'django-facetools',
    version = facetools.__version__,
    description = "Django Facetools provides a set of features to ease development of Facecbook canvas apps in a Django project.",
    long_description = get_long_description(),
    author = "Eric Palakovich Carr",
    author_email = "carreric@gmail.com",
    license = "MIT",
    url = "https://github.com/bigsassy/django-facetools",
    packages = [
        'facetools',
        'facetools.integrations',
        'facetools.management',
        'facetools.management.commands',
        'facetools.migrations',
        'facetools.templatetags',
        'facetools.test',
    ],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Topic :: Software Development :: Testing',
    ]
)



import types

from django.test.testcases import TestCase
from django.conf import settings
from facetools.models import TestUser
from facetools.common import _create_signed_request
from facetools.test import TestUserNotLoaded
from facetools.signals import sync_facebook_test_user, setup_facebook_test_client
from facetools.common import _get_facetools_test_fixture_name

# TODO: Add class that subclasses TransactionTestCase as well

class FacebookTestCase(TestCase):
    """
    TestCase which makes it possible to test views when the FacebookMiddleware
    and SyncFacebookUser middlewares are activated.  Must use the Client
    attached to this object (i.e. self.client).
    """
    facebook_test_user = None

    def _pre_setup(self):
        if self.facebook_test_user:
            if type(self.facebook_test_user) not in [str, unicode]:
                raise Exception("facebook_test_user variable must be a string (found a %s)" % type(self.facebook_test_user))

            app_name = get_app_name_from_test_case(type(self).__module__)
            facetools_fixture_name = _get_facetools_test_fixture_name(app_name)

            if not hasattr(self, 'fixtures'):
                self.fixtures = []
            if facetools_fixture_name not in self.fixtures:
                self.fixtures.append(facetools_fixture_name)
            super(FacebookTestCase, self)._pre_setup()

            # Make sure anybody that needs to sync their models loaded from fixtures
            # has a chance to do so now that the refreshed user test data is available.
            try:
                for test_user in TestUser.objects.all():
                    sync_facebook_test_user.send(sender=None, test_user=test_user)

                # Allow code to configure the test client so it has a signed request
                # of the specified test user for each request
                self.test_user = TestUser.objects.get(name=self.facebook_test_user)
                setup_facebook_test_client.send(sender=None, client=self.client, signed_request=_create_signed_request(
                    settings.FACEBOOK_APPLICATION_SECRET_KEY,
                    self.test_user.facebook_id,
                    oauth_token=self.test_user.access_token,
                ))
            except TestUser.DoesNotExist:
                raise TestUserNotLoaded("Test user %s hasn't been loaded via the %s fixture (did you run sync_facebook_test_users?)" %
                                        (self.facebook_test_user, facetools_fixture_name))
        else:
            super(FacebookTestCase, self)._pre_setup()

def get_app_name_from_test_case(module_path_string):
    """
    Gets thet Django app from the __class__ attribute of a TestCase in a Django app.
    class_string should look something like this: 'facetools_tests.tests.test_test_module'
    """
    packages = module_path_string.split(".")
    try:
        tests_location = packages.index("tests")
    except ValueError:
        raise ValueError("Couldn't find tests module in %s (are you running this test from tests.py or a tests package in your Django app?)" % module_path_string)
    if tests_location == 0:
        raise ValueError("Facetools doesn't support Django app's with a name of 'tests', or it failed to find the Django app name out of %s" % module_path_string)
    app_name = packages[tests_location - 1]
    if app_name not in settings.INSTALLED_APPS:
        raise ValueError("Facetools didn't find %s among INSTALLED_APPS. (app name pulled from %s)" % (app_name, module_path_string))
    return app_name
